/********************************************************************************
 Author : 

 Date : November 2014

 File name :  AD564.c

 Description :	 Define the different modes AD5664.

 Hardware plateform : 	AD5664 & ADuCM360
********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ADuCM360.h>

#include "DioLib.h"
#include "ClkLib.h"
#include "WdtLib.h"
#include "RstLib.h"
#include "spilib.h"
#include "UrtLib.h"

#include "AD5664.h"
#include "ADUCM360Driver.h"

extern unsigned char ucTxComplete;
extern unsigned long ulRxData;
long int DelayVal = 20;

void delay1 (long int length)
{
   while (length >0)
	 { 
		 length--;
	 }
}

void AD5664INIT(void)
{
	/*Configure P1.3 as output Pin*/
	DioCfgPin(pADI_GP1,PIN3,0);
	DioOenPin(pADI_GP1,PIN3,1);
	DioSet (pADI_GP1, 0x08);
	/*End of input pins configuration*/
	
	/*Configure P1.0, P1.1, and P1.2 as input Pins*/
	DioCfgPin(pADI_GP1,PIN0,0);
	DioCfgPin(pADI_GP1,PIN1,0);
	DioCfgPin(pADI_GP1,PIN2,0);
	
	DioOenPin(pADI_GP1,PIN0,0);
	DioOenPin(pADI_GP1,PIN1,0);
	DioOenPin(pADI_GP1,PIN2,0);	
	
	SPI0INIT();
	UARTINIT ();
}

 void AD5664_WriteToInputReg(unsigned long ulDACValue, unsigned long Address)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	 
	DioSet (pADI_GP1, 0x08);
	 
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x00)<<19) | Address;
	ulValue = (ulDACValue & 0XFFFF);
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
  delay1 (DelayVal);
 } 
 
void AD5664_UpdateDac(unsigned long Address)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x01)<<19) | Address;
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
void AD5664_WriteToInputRegAndUpdateDACs(unsigned long ulDACValue, unsigned long Address)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x02)<<19) | Address;
	ulValue = (ulDACValue & 0XFFFF);
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
void AD5664_WriteToInputRegDAC(unsigned long ulDACValue, unsigned long Address)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	 DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x03)<<19) | Address;
	ulValue = (ulDACValue & 0XFFFF);
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
void AD5664_PowerDown(unsigned int Mode, unsigned int PDAC)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	 DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x04)<<19);
	ulValue = Mode | PDAC;
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
 void AD5664_Reset(unsigned int Mode)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x05)<<19);
	ulValue = Mode;
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
 void AD5664_LDACRegSetUp(unsigned int LDACMode)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x06)<<19);
	ulValue = LDACMode;
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 } 
 
 void AD5664_InternalRefSetUp(unsigned int REF_Mode)
 {
	unsigned long ulCMD = 0;
	unsigned long ulValue = 0;
	DioSet (pADI_GP1, 0x08);
	SpiFifoFlush(pADI_SPI0,SPICON_TFLUSH_EN,SPICON_RFLUSH_DIS);
	
	ulCMD = ((0x07)<<19);
	ulValue = REF_Mode;
	ulValue |= ulCMD;					 				 // Append Command to bits 23-16.
	
	 DioClr (pADI_GP1, 0x08);
	//Transmit ulValue through SPI0
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 16));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue >> 8));
	SpiTx(pADI_SPI0,(unsigned char)(ulValue));
	 
  ucTxComplete = 0;
	while (ucTxComplete == 0) {}
	ucTxComplete = 0;
		
	delay1 (DelayVal);
	DioSet (pADI_GP1, 0x08);
	delay1 (DelayVal);
 }
