#include <stdio.h>
#include <string.h>
#include <ADuCM360.h>

#include "..\common\ClkLib.h"
#include "..\common\WdtLib.h"
#include "..\common\IntLib.h"
#include "..\common\DioLib.h"
#include "..\common\SpiLib.h"
#include "..\common\RstLib.h"
#include "..\common\UrtLib.h"
#include "AD5664.h"
#include "ADUCM360Driver.h"

void delay(long int);
void wait(void);

// SPI variables
unsigned char ucTxComplete = 0;                  // Flag used to indicate SPI transfer complete
unsigned long ulRxData = 0;                      // Used to read SPI1 values
unsigned long ulDacVal = 0;                      // Used to readback AD5683 DAC value through SPI1

// UART-based external variables
unsigned char ucTxBufferEmpty  = 0;				       // Used to indicate that the UART Tx buffer is empty
unsigned char szData[64] = "";					         // Used to store string before printing to UART
unsigned char nLen = 0;
unsigned char i = 0;
unsigned char ucWaitForUart = 0;				         // Used by calibration routines to wait for user input
volatile unsigned char bSendResultToUART = 0;

void SendResultToUART(void);

unsigned long SentData;


int main (void)
{
	
   WdtCfg(T3CON_PRE_DIV1,T3CON_IRQ_EN,T3CON_PD_DIS);  // Disable Watchdog timer resets
   
	//Disable clock to unused peripherals
	 ClkDis(CLKDIS_DISI2CCLK|CLKDIS_DISPWMCLK|CLKDIS_DIST0CLK|CLKDIS_DIST1CLK|CLKDIS_DISDACCLK|CLKDIS_DISDMACLK|CLKDIS_DISADCCLK);
   ClkCfg(CLK_CD0,CLK_HF,CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);            // Select CD0 for CPU clock - 16Mhz clock
	 ClkSel(CLK_CD0,CLK_CD7,CLK_CD0,CLK_CD7);      											// Select CD0 for SPI1 System clock
	
	 AD5664INIT ();
	 UARTINIT ();
	 
	 NVIC_EnableIRQ(SPI0_IRQn); 																				// Enable SPI1 interrupt sources
	 NVIC_EnableIRQ(UART_IRQn);
	 
	 SentData = 0x0000;
	 delay(0x0080);
	 AD5664_PowerDown (NORMAL, PDACA|PDACB|PDACC|PDACD);
	
  while(1)
	{	
		AD5664_WriteToInputRegAndUpdateDACs(SentData, DACall);		
		SendResultToUART();
	}                       
}

// Simple Delay routine
void wait ()
{
	while (ucTxComplete == 0){}                   // Wait for SPI1 interrupt
   ucTxComplete = 0;
	 delay(10);        													 // Wait for 150 ns after reset
		 
}
void delay (long int length)
{
   while (length >0)
      length--;
}


void SPI0_Int_Handler ()
{
	unsigned char uiSPI0STA = 0;

	uiSPI0STA = SpiSta(pADI_SPI0);
	if ((uiSPI0STA & SPI0STA_RXOF) == SPI0STA_RXOF)// SPI0 Rx Overflow IRQ
	{
	 	
	}
	if ((uiSPI0STA & SPI0STA_RX) == SPI0STA_RX)	 	 // SPI0 Rx IRQ
	{

	}
	if ((uiSPI0STA & SPI0STA_TX) == SPI0STA_TX)	 	 // SPI0 Tx IRQ
	{
		ucTxComplete = 1;
		ulRxData = SpiRx(pADI_SPI0);
		ulRxData = (ulRxData << 8);
		ulRxData |= SpiRx(pADI_SPI0);
		ulRxData = (ulRxData << 8);
		ulRxData |= SpiRx(pADI_SPI0);
	}
	if ((uiSPI0STA & SPI0STA_TXUR) == SPI0STA_TXUR)// SPI0 Tx underflow IRQ
	{

	}
}
//
void SendString (void)
{
   for ( i = 0 ; i < nLen ; i++ )
	{
  		 ucTxBufferEmpty = 0;
		   UrtTx(pADI_UART,szData[i]);
  		 while (ucTxBufferEmpty == 0)
  		 {
  		 }
	}
} 

void SendResultToUART(void)
{
	sprintf ( (char*)szData, "DAC Data: %X \r\n",(SentData-2048) );                          
	nLen = strlen((char*)szData);
	if (nLen <64)
 		SendString();
}


void UART_Int_Handler ()
{
   volatile unsigned char ucCOMSTA0 = 0;
   volatile unsigned char ucCOMIID0 = 0;

   ucCOMIID0 = UrtIntSta(pADI_UART);   // Read UART Interrupt ID register
   if ((ucCOMIID0 & 0x2) == 0x2)       // Transmit buffer empty
   {
      ucTxBufferEmpty = 1;
   }
   if ((ucCOMIID0 & 0x4) == 0x4)       // Receive byte
   {
      ucWaitForUart = 0;
   }
} 
